@@ -4,7 +4,7 @@ import functools
 import ply.lex
 
 from .parser_base import Parser
-from .errors import KarelSyntaxError, TimeoutError
+from .errors import KarelSyntaxError, TimeoutError, ExecutorSyntaxException
 
 
 def make_token(type, value=None):
@@ -461,31 +461,31 @@ class KarelForSynthesisParser(Parser):
 
 
 type_to_list_fn = {
-    "run": lambda v: ("DEF", "run", "m(") + tree_to_tokens(v["body"]) + ("m)",),
+    "run": lambda v: ("DEF", "run", "m(") + unparse(v["body"]) + ("m)",),
     "if": lambda v: ("IF", "c(")
-    + tree_to_tokens(v["cond"])
+    + unparse(v["cond"])
     + ("c)", "i(")
-    + tree_to_tokens(v["body"])
+    + unparse(v["body"])
     + ("i)",),
     "ifElse": lambda v: ("IFELSE", "c(")
-    + tree_to_tokens(v["cond"])
+    + unparse(v["cond"])
     + ("c)", "i(")
-    + tree_to_tokens(v["ifBody"])
+    + unparse(v["ifBody"])
     + ("i)", "ELSE", "e(")
-    + tree_to_tokens(v["elseBody"])
+    + unparse(v["elseBody"])
     + ("e)",),
     "while": lambda v: ("WHILE", "c(")
-    + tree_to_tokens(v["cond"])
+    + unparse(v["cond"])
     + ("c)", "w(")
-    + tree_to_tokens(v["body"])
+    + unparse(v["body"])
     + ("w)",),
     "repeat": lambda v: ("REPEAT",)
-    + tree_to_tokens(v["times"])
+    + unparse(v["times"])
     + ("r(",)
-    + tree_to_tokens(v["body"])
+    + unparse(v["body"])
     + ("r)",),
     "count": lambda v: ("R={:d}".format(v["value"]),),
-    "not": lambda v: ("not", "c(") + tree_to_tokens(v["cond"]) + ("c)",),
+    "not": lambda v: ("not", "c(") + unparse(v["cond"]) + ("c)",),
 }
 for k in (
     KarelForSynthesisParser.conditional_functions
@@ -494,7 +494,14 @@ for k in (
     type_to_list_fn[k] = lambda v, k=k: (k,)
 
 
-def tree_to_tokens(node):
+def unparse(node):
     if isinstance(node, list):
-        return tuple(token for item in node for token in tree_to_tokens(item))
+        return tuple(token for item in node for token in unparse(item))
     return type_to_list_fn[node["type"]](node)
+
+
+def parse(program):
+    try:
+        return KarelForSynthesisParser(build_tree=True).parse(program)
+    except KarelSyntaxError:
+        raise ExecutorSyntaxException
